PUBKEY_HASH_DICT = {
    "api.protonvpn.ch": [
        "IEwk65VSaxv3s1/88vF/rM8PauJoIun3rzVCX5mLS3M=",
        "drtmcR2kFkM8qJClsuWgUzxgBkePfRCkRpqUesyDmeE=", 
        "YRGlaY0jyJ4Jw2/4M8FIftwbDIQfh8Sdro96CeEel54=", 
        "AfMENBVvOS8MnISprtvyPsjKlPooqh8nMB/pvCrpJpw="
    ],
    "protonvpn.com": [
        "+0dMG0qG2Ga+dNE8uktwMm7dv6RFEXwBoBjQ43GqsQ0=",
        "8joiNBdqaYiQpKskgtkJsqRxF7zN0C0aqfi8DacknnI=",
        "JMI8yrbc6jB1FYGyyWRLFTmDNgIszrNEMGlgy972e7w=",
        "Iu44zU84EOCZ9vx/vz67/MRVrxF1IO4i4NIa8ETwiIY="
    ]
    
}